function [ model ] = adaboost_train( X, T, M )
%ADABOOST_TRAIN Summary of this function goes here
%   X - Nxdim matrix, N observations with dim properties each
%   T - Nx1 vector, the true labels of the observations
%   M - scalar, the number of basis functions to use
%
%   model.basisFuns - the trained weak classifiers
%   model.funsWeights - the weights of the weak classifiers

N = size(X,1);

% Initialize the weighting coefficients of the samples
weights = ones(N,1)/N;


epsilon = zeros(M,1);
alpha = zeros(M,1);
basisFunctions = cell(M,1);

for m=1:M
    % Fit a classifier y_m(x) to the training data by minimizing 
    % the weighted error function
    basisFunctions{m} = decisionstump_fit( X, T, weights );
    Ym = decisionstump_predict( X, basisFunctions{m} );
    
    missIdx = (Ym ~= T);
    weightedError = sum(weights .* missIdx);
    
    
    % Evaluate the quantities epsilon_m and alpha_m 
    epsilon(m) = weightedError / sum(weights); 
    
    % Avoid division with zero 
    if epsilon(m) == 0, epsilon(m) = eps; end   
    alpha(m) = log((1-epsilon(m)) / epsilon(m));
    
    % Update the data weighting coefficients
    %weightsOld = weights;
    weights = weights .* exp( alpha(m) * missIdx );
end

model.basisFuns = basisFunctions;
model.funsWeights = alpha;


end

